@@ -23,6 +23,7 @@ class GraphicalIndividual():
     __birth_date_ov = None
     # ordinal value of the death date
     __death_date_ov = None
+    weight = 1
 
     def __init__(self, instances, individual_id):
         self.items = []
@@ -32,8 +33,13 @@ class GraphicalIndividual():
         self.individual.graphical_representations.append(self)
         self.debug_label = ""
         self.g_id = None
+
+        # in ancestor charts children can be added, which are strongly connected, but
+        # since they are a dead end, not a root, they cannot be used for placement
         self.qualified_for_placement = True
 
+        self.special_properties = {}
+
         self.ancestor_chart_parent_family_placement = None, None
 
     def __repr__(self):
@@ -367,7 +373,7 @@ class GraphicalIndividual():
             return True
         return False
 
-    def set_position_vector(self, x_position, gr_family, this_is_the_parent_family=False):
+    def set_position_vector(self, x_position, gr_family, this_is_the_parent_family=False, overrule_ov=None):
         if gr_family:
             g_id = gr_family.g_id
             if gr_family.marriage:
@@ -377,13 +383,15 @@ class GraphicalIndividual():
         else:
             g_id = None
             ov = self.birth_date_ov
+        if overrule_ov:
+            ov = overrule_ov
         if not self._x_position:
             self._x_position = {}
         if g_id not in self._x_position:
             self._x_position[g_id] = (
                 (ov, x_position, gr_family, this_is_the_parent_family))
         _x_position = {}
-        _x_position.update(dict(sorted(self._x_position.items(), key=lambda t: t[1])))
+        _x_position.update(dict(sorted(self._x_position.items(), key=lambda t: t[1][0])))
         self._x_position = _x_position
 
     x_position = property(get_position_dict, set_position_vector)
@@ -509,6 +517,12 @@ class GraphicalIndividual():
         return gr_children
 
     def get_all_ancestors(self):
+        """
+        get all ancestors places over this individual
+
+        Returns:
+            list: list of ancestors
+        """
         gr_ancestors = []
         strongly_connected_parent_family, strongly_connected_spouse_family = self.ancestor_chart_parent_family_placement
         if strongly_connected_parent_family:
@@ -520,3 +534,12 @@ class GraphicalIndividual():
                 gr_ancestors += [(strongly_connected_parent_family, strongly_connected_parent_family.gr_wife)]
                 gr_ancestors += strongly_connected_parent_family.gr_wife.get_all_ancestors()
         return gr_ancestors
+
+    def get_all_descendants(self):
+        gr_descendants = []
+        for vm in self.visible_marriages:
+            for vc in vm.visible_children:
+                if not self.connected_parent_families or vm.descendant_chart_parent_family_placement == self.connected_parent_families[0]:
+                    gr_descendants += [(vm, vc)]
+                    gr_descendants += vc.get_all_descendants()
+        return gr_descendants
